@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Cone_spanners_2/include/CGAL/Cone_spanners_2/Less_by_direction_2.h $
+// $Id: Less_by_direction_2.h 26c857a 2018-06-13T15:11:45+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Authors: Weisheng Si, Quincy Tse
@@ -57,9 +58,10 @@ namespace CGAL {
  *
  */
 template <typename Kernel_, typename Graph_>
-class  Less_by_direction_2 : public std::binary_function <typename Graph_::vertex_descriptor,
-        typename Graph_::vertex_descriptor, bool> {
-
+class Less_by_direction_2
+  : public CGAL::cpp98::binary_function <typename Graph_::vertex_descriptor,
+                                         typename Graph_::vertex_descriptor, bool>
+{
 public:
     // typedef for C++11 - doesn't hurt to also have for C++98
     typedef typename Graph_::vertex_descriptor first_argument_type;
