@@ -16,8 +16,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Polygon/include/CGAL/Polygon_2.h $
+// $Id: Polygon_2.h b4a0ee0 2019-03-15T20:57:03+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Geert-Jan Giezeman <geert@cs.uu.nl>
@@ -30,11 +31,12 @@
 #ifndef CGAL_POLYGON_2_H
 #define CGAL_POLYGON_2_H
 
-#include <CGAL/basic.h>
+#include <CGAL/config.h>
 #include <vector>
 #include <list>
 #include <iterator>
 
+#include <CGAL/algorithm.h>
 #include <CGAL/circulator.h>
 #include <CGAL/enum.h>
 
@@ -47,7 +49,7 @@
 
 namespace CGAL {
 
-/// \ingroup PkgPolygon2
+/// \ingroup PkgPolygon2Ref
 /// The class Polygon_2 implements polygons. The Polygon_2 is
 /// parameterized by a traits class and a container class.  The latter
 /// can be any class that fulfills the requirements for an STL
@@ -120,26 +122,24 @@ class Polygon_2 {
     ///
     /// @{
 
-    /// 
+    /// vertex iterator type
     typedef typename Container::iterator       Vertex_iterator;
 
 
     //typedef typename Container::const_iterator Vertex_const_iterator; ??
 
 #ifdef DOXYGEN_RUNNING
+  /// vertex circulator type
   typedef unspecified_type Vertex_circulator;
-    typedef unspecified_type Edge_const_iterator;
-
-    typedef unspecified_type Edge_const_circulator;
-#else 
-    typedef Vertex_const_circulator            Vertex_circulator;
-    /// 
-    typedef Polygon_2_edge_iterator<Traits_P,Container_P>
-            Edge_const_iterator;
-
-    /// 
-    typedef Polygon_2_const_edge_circulator<Traits_P,Container_P>
-            Edge_const_circulator;
+  /// edge circulator type
+  typedef unspecified_type Edge_const_iterator;
+  /// edge circular type
+  typedef unspecified_type Edge_const_circulator;
+#else
+    typedef Vertex_const_circulator Vertex_circulator;
+    typedef Polygon_2_edge_iterator<Traits_P,Container_P> Edge_const_iterator;
+    typedef Polygon_2_const_edge_circulator<Traits_P,
+                                            Container_P> Edge_const_circulator;
 #endif // DOXYGEN_RUNNING    
     /// @}
 
@@ -164,7 +164,13 @@ class Polygon_2 {
       // Sun STL switches off member templates for binary backward compat.
       std::copy(first, last, std::back_inserter(d_container));
     }
-
+  
+#ifndef CGAL_CFG_NO_CPP0X_DELETED_AND_DEFAULT_FUNCTIONS
+#ifndef DOXYGEN_RUNNING
+  Polygon_2& operator=(const Polygon_2&)=default;
+#endif
+#endif
+  
     /// @}
 
     /// \name Modifiers
@@ -175,10 +181,12 @@ class Polygon_2 {
     void set(Vertex_iterator i, const Point_2& q)
      { *i = q; }
 
+    /// \cond
     void set(Polygon_circulator<Container>const &i, const Point_2& q)
      {
        *i.mod_iterator() = q;
      }
+    /// \endcond
 
     /// Inserts the vertex `q` before `i`. The return value points to
     /// the inserted vertex.
@@ -187,6 +195,8 @@ class Polygon_2 {
         return d_container.insert(i,q);
       }
 
+    /// Inserts the vertex `q` before `i`. The return value points to
+    /// the inserted vertex.
     Vertex_iterator insert(Vertex_circulator i, const Point_2& q)
       {
         return d_container.insert(i.mod_iterator(),q);
@@ -194,13 +204,16 @@ class Polygon_2 {
 
     /// Inserts the vertices in the range `[first, last)`
     /// before `i`.  The value type of points in the range
-    /// `[first,last)} must be \ccStyle{Point_2`.
+    /// `[first,last)` must be `Point_2`.
     template <class InputIterator>
     void insert(Vertex_iterator i,
                 InputIterator first,
                 InputIterator last)
       { d_container.insert(i, first, last); }
 
+    /// Inserts the vertices in the range `[first, last)`
+    /// before `i`.  The value type of points in the range
+    /// `[first,last)` must be `Point_2`.
     template <class InputIterator>
     void insert(Vertex_circulator i,
                 InputIterator first,
@@ -217,6 +230,7 @@ class Polygon_2 {
         return d_container.erase(i);
       }
 
+    /// Erases the vertex pointed to by `i`.
     Vertex_circulator erase(Vertex_circulator i)
       {
         return Vertex_circulator(&d_container,
@@ -313,7 +327,7 @@ class Polygon_2 {
       return orientation_2(d_container.begin(), d_container.end(), traits);
     }
 
-    /// Returns `POSITIVE_SIDE`, or `NEGATIVE_SIDE`,
+    /// Returns `ON_POSITIVE_SIDE`, or `ON_NEGATIVE_SIDE`,
     /// or `ON_ORIENTED_BOUNDARY`, depending on where point
     /// `q` is. 
     /// \pre `p.is_simple()`.
@@ -387,31 +401,32 @@ class Polygon_2 {
     /// @}
 
 
-    /// \name 
+    /// \name Convenience Orientation Functions
     /// For convenience we provide the following Boolean functions:
     /// @{
 
+    /// returns `orientation() == COUNTERCLOCKWISE`
     bool is_counterclockwise_oriented() const
       { return orientation() == COUNTERCLOCKWISE; }
-
+    /// returns `orientation() == CLOCKWISE`
     bool is_clockwise_oriented() const
       { return orientation() == CLOCKWISE; }
-
+    /// returns `orientation() == COLLINEAR`
     bool is_collinear_oriented() const
       { return orientation() == COLLINEAR; }
-
+    /// returns `oriented_side(q) == ON_POSITIVE_SIDE`
     bool has_on_positive_side(const Point_2& q) const
       { return oriented_side(q) == ON_POSITIVE_SIDE; }
-
+    /// returns `oriented_side(q) == ON_NEGATIVE_SIDE`
     bool has_on_negative_side(const Point_2& q) const
       { return oriented_side(q) == ON_NEGATIVE_SIDE; }
-
+    /// returns `bounded_side(q) == ON_BOUNDARY`
     bool has_on_boundary(const Point_2& q) const
       { return bounded_side(q) == ON_BOUNDARY; }
-
+    /// returns `bounded_side(q) == ON_BOUNDED_SIDE`
     bool has_on_bounded_side(const Point_2& q) const
       { return bounded_side(q) == ON_BOUNDED_SIDE; }
-
+    /// returns `bounded_side(q) == ON_UNBOUNDED_SIDE`
     bool has_on_unbounded_side(const Point_2& q) const
       { return bounded_side(q) == ON_UNBOUNDED_SIDE; }
 
@@ -423,7 +438,10 @@ class Polygon_2 {
 
     /// Returns a (const) reference to the `i`-th vertex.
     const Point_2& vertex(std::size_t i) const
-      { return *(d_container.begin() + i); }
+      {
+        CGAL_precondition( i < d_container.size() );
+        return *(cpp11::next(d_container.begin(), i));
+      }
 
 
     /// Returns a (const) reference to the `i`-th vertex.
@@ -432,7 +450,7 @@ class Polygon_2 {
 
     /// Returns the `i`-th edge.
     Segment_2 edge(std::size_t i) const
-      { return *(edges_begin() + i); }
+      { return *(cpp11::next(edges_begin(), i)); }
 
     /// @}
 
@@ -471,7 +489,7 @@ class Polygon_2 {
 /// Test for equality: two polygons are equal iff there exists a
 /// cyclic permutation of the vertices of `p2` such that they are
 /// equal to the vertices of `p1`. Note that the template argument
-/// `Container` of `p1` and `p2` may be different.
+/// `%Container` of `p1` and `p2` may be different.
 /// \memberof Polygon_2
 template <class Traits_P, class Container1_P, class Container2_P>
 bool operator==( const Polygon_2<Traits_P,Container1_P> &p1,
@@ -498,18 +516,17 @@ transform(const Transformation& t, const Polygon_2<Traits_P,Container_P>& p);
 /// followed by the output of the coordinates of the vertices.
 /// @{
 
-/// Inserts the polygon `p` into the stream `os`. \pre The insert
-/// operator must be defined for `Point_2`.
+/// Reads a polygon from stream `is` and assigns it to `p`.
+/// \pre The extract operator must be defined for `Point_2`.
 /// \memberof Polygon_2
 template <class Traits_P, class Container_P>
 std::istream &operator>>(std::istream &is, Polygon_2<Traits_P,Container_P>& p);
 
-/// Reads a polygon from stream `is` and assigns it
-/// to `p`. \pre The extract operator must be defined for `Point_2`.
+/// Inserts the polygon `p` into the stream `os`.
+/// \pre The insert operator must be defined for `Point_2`.
 /// \memberof Polygon_2
 template <class Traits_P, class Container_P>
-std::ostream
-&operator<<(std::ostream &os, const Polygon_2<Traits_P,Container_P>& p);
+std::ostream &operator<<(std::ostream &os, const Polygon_2<Traits_P,Container_P>& p);
 
 /// @} // IO
 
